// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) 2008-2011 ZenJu (zhnmju123 AT gmx.de)                    *
// **************************************************************************
//
#ifndef UNITTEST_H_INCLUDED
#define UNITTEST_H_INCLUDED

#include <gtest/gtest.h>
#include <wx/datetime.h>
#include "../status_handler.h"
#include "../../shared/zstring.h"
#include "../../file_hierarchy.h"


//predicate-formatter for wxDateTime
testing::AssertionResult assertEqual(const char* a_expr, const char* b_expr, const wxDateTime& a, const wxDateTime& b);
//predicate-formatter for wxULongLong
testing::AssertionResult assertEqual(const char* a_expr, const char* b_expr, const wxULongLong& a, const wxULongLong& b);
//predicate-formatter for wxString
testing::AssertionResult assertEqual(const char* a_expr, const char* b_expr, const wxString& a, const wxString& b);


unsigned int getTotalElementNo(const ffs3::HierarchyObject& hierObj);

class DummyStatusUpdater : public StatusHandler //mock object
{
public:
    DummyStatusUpdater() {}
    ~DummyStatusUpdater() {}

    virtual ErrorHandler::Response reportError(const wxString& text)
    {
        throw text;
        return ErrorHandler::IGNORE_ERROR; //dummy return value
    }

    virtual void reportFatalError(const wxString& errorMessage)
    {
        throw errorMessage;
    }

    virtual void reportWarning(const wxString& warningMessage, bool& warningActive)
    {
        if (warningActive)
            throw warningMessage;
    }


    void reportInfo(const Zstring& text) {}
    void initNewProcess(int objectsTotal, wxLongLong dataTotal, Process processID) {}
    void updateProcessedData(int objectsProcessed, wxLongLong dataProcessed) {}
    void forceUiRefresh() {}

private:
    void abortThisProcess() {}
};


#endif // UNITTEST_H_INCLUDED
