// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) 2008-2011 ZenJu (zhnmju123 AT gmx.de)                    *
// **************************************************************************
//
#ifndef RECYCLER_DLL_H
#define RECYCLER_DLL_H

#ifdef FILE_OP_DLL_EXPORTS
#define FILE_OP_DLL_API extern "C" __declspec(dllexport)
#else
#define FILE_OP_DLL_API extern "C" __declspec(dllimport)
#endif


namespace fileop
{
/*--------------
  |declarations|
  --------------*/

//COM needs to be initialized before calling any of these functions! CoInitializeEx/CoUninitialize

FILE_OP_DLL_API
bool moveToRecycleBin(const wchar_t* fileNames[],
                      size_t         fileNo); //size of fileNames array

FILE_OP_DLL_API
bool copyFile(const wchar_t* sourceFile,
              const wchar_t* targetFile);

//if any of the functions above returns 'false', this message returns last error
FILE_OP_DLL_API
void getLastError(wchar_t* errorMessage, size_t errorBufferLen);

/*----------
  |typedefs|
  ----------*/
typedef bool (*MoveToRecycleBinFct)(const wchar_t* fileNames[], size_t fileNo);
typedef bool (*CopyFileFct)(const wchar_t* sourceFile, const wchar_t* targetFile);
typedef void (*GetLastErrorFct)(wchar_t* errorMessage, size_t errorBufferLen);

/*--------------
  |symbol names|
  --------------*/
//(use const pointers to ensure internal linkage)
const char* const moveToRecycleBinFctName = "moveToRecycleBin";
const char* const copyFileFctName         = "copyFile";
const char* const getLastErrorFctName     = "getLastError";
}



#endif //RECYCLER_DLL_H
