// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) 2008-2011 ZenJu (zhnmju123 AT gmx.de)                    *
// **************************************************************************
//

#include <iostream>
#include <fstream>
#include "shadow.h"
#define WIN32_LEAN_AND_MEAN
#include "windows.h"
#include "ObjBase.h"

using namespace shadow;

class CloseHandleOnExit
{
public:
    CloseHandleOnExit(HANDLE fileHandle) : fileHandle_(fileHandle) {}
    ~CloseHandleOnExit()
    {
        CloseHandle(fileHandle_);
    }
private:
    HANDLE fileHandle_;
};


int main(int argc, wchar_t* argv[])
{
    //create test file
    const wchar_t* testFileName = L"C:\\locked.txt";
    std::ofstream testFile(testFileName);
    testFile << "sample text" << std::endl;
    testFile.close();

    //obtain exclusive lock on test file
    HANDLE hr = CreateFile(testFileName,
                           GENERIC_READ,
                           0, //no sharing!
                           NULL,
                           OPEN_EXISTING,
                           FILE_FLAG_BACKUP_SEMANTICS,
                           NULL);
    if (hr == INVALID_HANDLE_VALUE)
    {
        std::cout << "error obtaining exclusive lock on test file: " << std::endl;
        return -1;
    }

    CloseHandleOnExit dummy(hr);

    //make shadow copy
    CoInitializeEx( NULL, 0 );

    wchar_t message[1000];
    wchar_t shadowVolName[1000];
    ShadowHandle backupHandle = NULL;
    if (createShadowCopy(L"C:\\", shadowVolName, 1000, &backupHandle, message, 1001))
    {
        //now copying should be possible using the shadow copy service
        std::wstring shadowFile(shadowVolName);
        shadowFile += L"\\locked.txt";
        CopyFile(shadowFile.c_str(),
                 L"C:\\copyLocked.txt",
                 true);

        //clean up
        releaseShadowCopy(backupHandle);
    }
    else
    {
        MessageBox(0, message, L"", 0);
    }

    CoUninitialize();

    return 0;
}

