// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) 2008-2011 ZenJu (zhnmju123 AT gmx.de)                    *
// **************************************************************************
//
#ifndef SHADOWCOPY_H
#define SHADOWCOPY_H

#ifdef SHADOWDLL_EXPORTS
#define SHADOWDLL_API extern "C" __declspec(dllexport)
#else
#define SHADOWDLL_API extern "C" __declspec(dllimport)
#endif


namespace shadow
{
/*--------------
  |declarations|
  --------------*/

//COM needs to be initialized before calling any of these functions! CoInitializeEx/CoUninitialize

typedef size_t ShadowHandle;

//volumeName must end with "\", while shadowVolName does not end with "\"
SHADOWDLL_API
bool createShadowCopy(const wchar_t* volumeName,      //[in]
                      wchar_t*		 shadowVolName,   //[out]
                      unsigned int   shadowBufferLen, //[in]
                      ShadowHandle*	 handle,          //[out]
                      wchar_t*		 errorMessage,    //[out]
                      unsigned int	 errorBufferLen); //[in]


//don't forget to release the backupHandle after shadow copy is not needed anymore!
SHADOWDLL_API
void releaseShadowCopy(ShadowHandle handle);


//##########################################################################################


/*----------
  |typedefs|
  ----------*/
typedef bool (*CreateShadowCopyFct)(const wchar_t* volumeName,
                                    wchar_t*	   shadowVolName,
                                    unsigned int   shadowBufferLen,
                                    ShadowHandle*  handle,
                                    wchar_t*	   errorMessage,
                                    unsigned int   errorBufferLen);

typedef void (*ReleaseShadowCopyFct)(ShadowHandle handle);

/*--------------
  |symbol names|
  --------------*/
//(use const pointers to ensure internal linkage)
const char* const createShadowCopyFctName  = "createShadowCopy";
const char* const releaseShadowCopyFctName = "releaseShadowCopy";
}



#endif //SHADOWCOPY_H
