// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) 2008-2011 ZenJu (zhnmju123 AT gmx.de)                    *
// **************************************************************************
//
#include <iostream>
#include <fstream>
#include <set>
#include <wx/wx.h>
#include <wx/filename.h>
#include <wx/dir.h>

using namespace std;

//reads all texts in _("dummy") into file %2 for translation (and preserves content including already translated lines)
//discards entries that are not imported newly via the files from directory %1

struct TranslationLine
{
    string original;
    string translation;

    bool operator>(const TranslationLine& b ) const
    {
        return (original > b.original);
    }
    bool operator<(const TranslationLine& b) const
    {
        return (original < b.original);
    }
    bool operator==(const TranslationLine& b) const
    {
        return (original == b.original);
    }
};


int main(int args, char** argv)
{
    //get input files (*.cpp and *.h files including sub-dirs of directory specified in arg[1]
    wxArrayString inputFiles;

    if (args == 3)
    {
        wxDir::GetAllFiles(argv[1], &inputFiles, "*.h");
        wxDir::GetAllFiles(argv[1], &inputFiles, "*.cpp");

        set<TranslationLine> textBufferInput;
        set<TranslationLine> textBufferOutput;
        TranslationLine currentLine;

        const int TEMP_SIZE = 100000;
        char temp[TEMP_SIZE];

        const string inOutFile = argv[2];

        //read inOutFile into buffer, if existing
        ifstream localize(inOutFile.c_str());
        if (localize)
        {
            int rowNumber = 0;
            while (localize.getline(temp, TEMP_SIZE))
            {
                //remove UTF-8 Byte Ordering Mark
                if (    temp[0] == static_cast<char>(0xEF) &&
                        temp[1] == static_cast<char>(0xBB) &&
                        temp[2] == static_cast<char>(0xBF))
                {
                    memmove(temp, temp + 3, TEMP_SIZE - 3);
                }

                if (rowNumber%2 == 0)  //  -> original text and translation alternate
                    currentLine.original = temp;
                else
                {
                    const string translationTxt = temp;
                    if (!translationTxt.empty()) //only add if translation is existing
                    {
                        currentLine.translation = translationTxt;
                        textBufferInput.insert(currentLine);
                    }
                }
                rowNumber++;
            }
            localize.close();
        }

        //read list of texts to exclude
        set<string> excludeList;
        ifstream exclude("exclude.txt");
        if (exclude)
        {
            while (exclude.getline(temp, TEMP_SIZE))
                excludeList.insert(temp);

            exclude.close();
        }

        bool notTranslatedTextsExist = false;

        set<TranslationLine>::iterator indexInput;
        for (unsigned int i = 0; i < inputFiles.GetCount(); ++i)
        {
            ifstream input(inputFiles[i]);
            if (input)
            {
                while (input.getline(temp, TEMP_SIZE))
                {
                    string tmpString(temp);
                    unsigned indexStart = 0;
                    while ((indexStart = tmpString.find("_(\"", indexStart)) != string::npos)
                    {
                        unsigned int indexEnd = 0;
                        if ((indexEnd = tmpString.find("\")", indexStart + 3)) != string::npos)
                        {
                            currentLine.original = tmpString.substr(indexStart + 3, indexEnd - indexStart - 3);

                            if (!currentLine.original.empty()) //do not add empty rows
                            {
                                //add element only if NOT in excludeList
                                if (excludeList.find(currentLine.original) == excludeList.end())
                                {
                                    //try to load entries from changing file (inOutFile)
                                    if ((indexInput = textBufferInput.find(currentLine)) != textBufferInput.end())
                                        currentLine.translation = indexInput->translation;
                                    else
                                    {
                                        currentLine.translation.clear();
                                        notTranslatedTextsExist = true;
                                    }

                                    textBufferOutput.insert(currentLine);
                                }
                            }

                            indexStart = indexEnd + 2;
                        }
                        else break;
                    }
                }
                input.close();
            }
            else
            {
                cout<<"Unable to read "<<inputFiles[i]<<"!"<<endl;
                system("pause");
                return -1;
            }
        }

        ofstream writeLocalize(inOutFile.c_str());
        for (set<TranslationLine>::iterator i = textBufferOutput.begin(); i != textBufferOutput.end(); ++i)
        {
            writeLocalize<<i->original.c_str()<<endl;
            writeLocalize<<i->translation.c_str()<<endl;
        }
        writeLocalize.close();


        cout<<"\nOperation successful! ";
        if (notTranslatedTextsExist)
            cout<<"\nTexts to be translated are existing!!!"<<endl<<endl;
        else
            cout<<"All OK!"<<endl<<endl;

        return 0;
    }
    else
    {
        cout<<"\nInvalid input parameters: Specify input directory to search (%1) and changing language file (%2)\n"<<endl;
        system("pause");
        return -1;
    }
}
