// **************************************************************************
// * This file is part of the FreeFileSync project. It is distributed under *
// * GNU General Public License: http://www.gnu.org/licenses/gpl.html       *
// * Copyright (C) 2008-2011 ZenJu (zhnmju123 AT gmx.de)                    *
// **************************************************************************
//
#include <iostream>
#include <fstream>
#include <sstream>
#include <cassert>

int main(int args, char* argv[])
{
    if (args != 3)
    {
        std::cout << "Please specify a source filename as %1 parameter and target as %2!" << std::endl;
        return -1;
    }

    //read input file into buffer, if existing
    std::ifstream inputFile(argv[1], std::ios::binary);
    if (!inputFile)
    {
        std::cout << "Error opening " << argv[1] << "!" << std::endl;
        return -1;
    }

    std::stringstream buffer(std::ios::out | std::ios::in | std::ios::binary);

    char temp[3];
    inputFile.read(temp, 3);
    if (inputFile.eof())  //end of file? fail bit is set in this case also!
    {
        std::cout << "No BOM found in file " << argv[1] << "!" << std::endl;

        buffer.write(temp, inputFile.gcount());
        assert(inputFile.gcount() < 3);
    }
    else if (inputFile.fail())
    {
        std::cout << "Error reading first bytes of " << argv[1] << "!" << std::endl;
        return -1;
    }
    else if (    temp[0] != static_cast<char>(0xEF) || //remove UTF-8 Byte Ordering Mark: first three bytes from inputFile are already skipped!
                 temp[1] != static_cast<char>(0xBB) ||
                 temp[2] != static_cast<char>(0xBF))
    {
        std::cout << "No BOM found in file " << argv[1] << "!" << std::endl;

        buffer.write(temp, inputFile.gcount());
        assert(inputFile.gcount() == 3);
    }

    //write rest...
    buffer << inputFile.rdbuf(); //input file may be equal to output file!

    inputFile.close();

    //write file without BOM
    std::ofstream outputfile(argv[2], std::ios::binary);
    if (!outputfile)
    {
        std::cout << "Error writing " << argv[2] << "!" << std::endl;
        return -1;
    }

    outputfile << buffer.rdbuf();

    std::cout << "\nOperation successful!" << std::endl;
    return 0;
}
